@@ -15,9 +15,9 @@
 from logging import getLogger, DEBUG, Formatter, Handler, addLevelName
 from logging.handlers import RotatingFileHandler
 from sys import platform
-from os import path, system
+from os import path, system, makedirs
 from time import time, sleep
-from argparse import ArgumentParser
+from argparse import ArgumentParser,SUPPRESS
 from json import load, dumps
 from uuid import uuid4
 from functools import wraps
@@ -33,35 +33,36 @@ from urllib.parse import unquote, urlparse
 from urllib3.exceptions import InsecureRequestWarning
 from bs4 import BeautifulSoup
 from tld import get_fld
-from requests import get, packages
+from requests import get, packages, Session
 from termcolor import colored
 from langdetect import detect
+from warnings import filterwarnings
+
+filterwarnings('ignore', category=RuntimeWarning, module='runpy')
+packages.urllib3.disable_warnings(category=InsecureRequestWarning)
 
 if platform == "win32":
     system("color")
 
-packages.urllib3.disable_warnings(category=InsecureRequestWarning)
-
 WEBSITES_ENTRIES = []
 SHARED_DETECTIONS = []
 GENERIC_DETECTION = []
 LOG = getLogger("social-analyzer")
 SITES_PATH = path.join(path.dirname(__file__), "data", "sites.json")
 LANGUAGES_PATH = path.join(path.dirname(__file__), "data", "languages.json")
-STRINGS_PAGES = recompile('captcha-info|Please enable cookies', IGNORECASE)
-STRINGS_TITLES = recompile('found|attention required|cloudflare', IGNORECASE)
-LANGUAGES_JSON = {}
+STRINGS_PAGES = recompile('captcha-info|Please enable cookies|Completing the CAPTCHA', IGNORECASE)
+STRINGS_TITLES = recompile('not found|blocked|attention required|cloudflare', IGNORECASE)
+STRINGS_META = recompile(r'regionsAllowed|width|height|color|rgba\(|charset|viewport|refresh|equiv', IGNORECASE)
+LANGUAGES_JSON = None
+SITES_DUMMY = None
 WORKERS = 15
 CUSTOM_MESSAGE = 51
 WAF = True
 
-with open(LANGUAGES_PATH) as f:
-    LANGUAGES_JSON = load(f)
-
 
 def delete_keys(in_object, keys):
-    ''' 
-    delete specific keys from object 
+    '''
+    delete specific keys from object
     '''
 
     for key in keys:
@@ -71,15 +72,15 @@ def delete_keys(in_object, keys):
 
 
 def clean_up_item(in_object, keys_str):
-    ''' 
+    '''
     delete specific keys from object (user input)
     '''
 
     with suppress(Exception):
         del in_object["image"]
-    if keys_str == "" or keys_str == None:
+    if keys_str == "" or keys_str is None:
         with suppress(Exception):
-            del in_object["text"]
+            pass
     else:
         for key in in_object.copy():
             if key not in keys_str:
@@ -89,7 +90,7 @@ def clean_up_item(in_object, keys_str):
 
 
 def get_language_by_guessing(text):
-    ''' 
+    '''
     guess language by text, this needs long text
     '''
 
@@ -101,7 +102,7 @@ def get_language_by_guessing(text):
 
 
 def get_language_by_parsing(source):
-    ''' 
+    '''
     guess language by parsing the lang tag
     '''
 
@@ -113,7 +114,7 @@ def get_language_by_parsing(source):
 
 
 def check_errors(on_off=None):
-    ''' 
+    '''
     wrapper function for debugging
     '''
 
@@ -133,12 +134,12 @@ def check_errors(on_off=None):
 
 
 class CustomHandler(Handler):
-    ''' 
+    '''
     custom stream handler
     '''
 
     def __init__(self, argv=None):
-        ''' 
+        '''
         int, user choices needed
         '''
 
@@ -146,7 +147,7 @@ class CustomHandler(Handler):
         self.argv = argv
 
     def emit(self, record):
-        ''' 
+        '''
         emit, based on user choices
         '''
 
@@ -165,7 +166,7 @@ class CustomHandler(Handler):
                                             empty_string = colored(empty_string.ljust(13, ' '), 'blue') + ": "
                                             for _item_key, _item_value in _item.items():
                                                 if self.argv.trim and _item_key == "content" and len(_item_value) > 50:
-                                                    empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value[:50]+"..", 'yellow'))
+                                                    empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value[:50] + "..", 'yellow'))
                                                 else:
                                                     empty_string += "{} : {} ".format(colored(_item_key, 'blue'), colored(_item_value, 'yellow'))
                                             print("{}".format(empty_string))
@@ -178,13 +179,13 @@ class CustomHandler(Handler):
 
 @check_errors(True)
 def setup_logger(uuid=None, file=False, argv=None):
-    ''' 
+    '''
     setup a logger for logs in the temp folder
     '''
 
     temp_folder = mkdtemp()
     if argv.output != "json":
-        print('[!] Temporary Logs Directory {}'.format(temp_folder))
+        print('[init] Temporary Logs Directory {}'.format(temp_folder))
     LOG.setLevel(DEBUG)
     LOG.addHandler(CustomHandler(argv))
     addLevelName(CUSTOM_MESSAGE, "CUSTOM")
@@ -196,7 +197,7 @@ def setup_logger(uuid=None, file=False, argv=None):
 
 @check_errors(True)
 def init_detections(detections):
-    ''' 
+    '''
     load websites_entries, shared_detections and generic_detection
     '''
 
@@ -209,7 +210,7 @@ def init_detections(detections):
 
 
 def get_website(site):
-    ''' 
+    '''
     extract domain from website
     '''
 
@@ -219,10 +220,9 @@ def get_website(site):
 
 
 def list_all_websites():
-    ''' 
+    '''
     list all the available websites' entries
     '''
-
     if len(WEBSITES_ENTRIES) > 0:
         for site in WEBSITES_ENTRIES:
             temp_value = get_fld(site["url"], fix_protocol=True)
@@ -232,14 +232,14 @@ def list_all_websites():
 
 @check_errors(True)
 def find_username_normal(req):
-    ''' 
+    '''
     main find usernames logic using ThreadPoolExecutor
     '''
 
     resutls = []
 
     def fetch_url(site, username, options):
-        ''' 
+        '''
         this runs for every website entry
         '''
 
@@ -270,16 +270,19 @@ def find_username_normal(req):
         }
 
         with suppress(Exception):
-            response = get(site["url"].replace("{username}", username), timeout=5, headers=headers, verify=False)
+            session = Session()
+            session.headers.update(headers)
+            response = session.get(site["url"].replace("{username}", username), timeout=5, verify=False)
             source = response.text
+            content = response.content
             answer = dict((k.lower(), v.lower()) for k, v in response.headers.items())
-            response.close()
+            session.close()
             temp_profile = {}
             temp_detected = {}
             detections_count = 0
 
             def check_url(url):
-                ''' 
+                '''
                 check if url is okay
                 '''
 
@@ -290,7 +293,7 @@ def find_username_normal(req):
                 return False
 
             def merge_dicts(temp_dict):
-                ''' 
+                '''
                 '''
 
                 result = {}
@@ -303,7 +306,7 @@ def find_username_normal(req):
                 return result
 
             def detect_logic(detections):
-                ''' 
+                '''
                 check for detections in website entry
                 '''
 
@@ -336,7 +339,7 @@ def find_username_normal(req):
                 return temp_profile, temp_detected, detections_count
 
             def detect():
-                ''' 
+                '''
                 main detect logic
                 '''
 
@@ -365,19 +368,25 @@ def find_username_normal(req):
             soup = None
 
             with suppress(Exception):
-                soup = BeautifulSoup(source, "html.parser")
+                soup = BeautifulSoup(content, "html.parser")
 
             with suppress(Exception):
-                [tag.extract() for tag in soup(["head", "title", "style", "script", "[document]"])]
-                temp_profile["text"] = soup.getText()
-                temp_profile["text"] = resub("\s\s+", " ", temp_profile["text"])
+                temp_text_arr = []
+                temp_text_list = []
+                soup = BeautifulSoup(content, "html.parser")
+                for item in soup.stripped_strings:
+                    if item not in temp_text_list:
+                        temp_text_list.append(item)
+                        temp_text_arr.append(repr(item).replace("'", ""))
+                temp_profile["text"] = " ".join(temp_text_arr)
+                temp_profile["text"] = resub(r"\s\s+", " ", temp_profile["text"])
             with suppress(Exception):
                 temp_profile["language"] = get_language_by_parsing(source)
                 if temp_profile["language"] == "unavailable":
                     temp_profile["language"] = get_language_by_guessing(temp_profile["text"])
             with suppress(Exception):
                 temp_profile["title"] = BeautifulSoup(source, "html.parser").title.string
-                temp_profile["title"] = resub("\s\s+", " ", temp_profile["title"])
+                temp_profile["title"] = resub(r"\s\s+", " ", temp_profile["title"])
 
             with suppress(Exception):
                 temp_matches = []
@@ -417,7 +426,9 @@ def find_username_normal(req):
                             temp_profile["title"] = "filtered"
                     if research(STRINGS_PAGES, temp_profile["text"]):
                         temp_profile["text"] = "filtered"
+                        temp_profile["title"] = "filtered"
                     if research(STRINGS_TITLES, temp_profile["title"]):
+                        temp_profile["text"] = "filtered"
                         temp_profile["title"] = "filtered"
 
             with suppress(Exception):
@@ -437,8 +448,9 @@ def find_username_normal(req):
                 if temp_profile["status"] == "good":
                     temp_meta_list = []
                     temp_for_checking = []
-                    for meta in soup.findAll('meta'):
-                        if meta not in temp_for_checking:
+                    soup = BeautifulSoup(content, "lxml")
+                    for meta in soup.find_all('meta'):
+                        if meta not in temp_for_checking and not research(STRINGS_META, str(meta)):
                             temp_for_checking.append(meta)
                             temp_mata_item = {}
                             add = True
@@ -517,7 +529,7 @@ def find_username_normal(req):
 
 @check_errors(True)
 def check_user_cli(argv):
-    ''' 
+    '''
     main cli logic
     '''
 
@@ -544,14 +556,14 @@ def check_user_cli(argv):
     resutls = find_username_normal(req)
 
     for item in resutls:
-        if item != None:
+        if item is not None:
             if item["method"] == "all":
                 if item["good"] == "true":
                     item = delete_keys(item, ["method", "good"])
                     item = clean_up_item(item, argv.options)
                     temp_detected["detected"].append(item)
                 else:
-                    item = delete_keys(item, ["found", "rate", "status", "method", "good", "extracted", "metadata"])
+                    item = delete_keys(item, ["found", "rate", "status", "method", "good", "text", "extracted", "metadata"])
                     item = clean_up_item(item, argv.options)
                     temp_detected["unknown"].append(item)
             elif item["method"] == "find":
@@ -560,7 +572,7 @@ def check_user_cli(argv):
                     item = clean_up_item(item, argv.options)
                     temp_detected["detected"].append(item)
             elif item["method"] == "get":
-                item = delete_keys(item, ["found", "rate", "status", "method", "good", "extracted", "metadata"])
+                item = delete_keys(item, ["found", "rate", "status", "method", "good", "text", "extracted", "metadata"])
                 item = clean_up_item(item, argv.options)
                 temp_detected["unknown"].append(item)
             else:
@@ -615,47 +627,89 @@ def check_user_cli(argv):
         print(dumps(temp_detected, sort_keys=True, indent=None))
 
 
+def load_file(name, path_to_check, url_download):
+    ret = None
+    try:
+        if path.exists(path_to_check) == False:
+            print("[init] Downloading {} from {}".format(name, url_download))
+            file = get(url_download, allow_redirects=True)
+            with open(path_to_check, 'wb') as f:
+                f.write(file.content)
+        if path.exists(path_to_check) == True:
+            print("[init] {} looks good!".format(name))
+            with open(path_to_check) as f:
+                ret = load(f)
+    except Exception as e:
+        print("[!] {} Does not exist! cannot be downloaded...".format(name))
+    return ret
+
+
 def msg():
-    ''' 
+    '''
     welcome message
     '''
 
-    if path.basename(__file__) == "__main__.py":
-        return """python -m social-analyzer --cli --mode 'fast' --username 'johndoe' --websites 'youtube pinterest tumblr' --output 'pretty'"""
-    return """python3 app.py --cli --mode 'fast' --username 'johndoe' --websites 'youtube pinterest tumblr' --output 'pretty'"""
+    return "\n[social-analyzer]"
 
 
-WEBSITES_ENTRIES = init_detections("websites_entries")
-SHARED_DETECTIONS = init_detections("shared_detections")
-GENERIC_DETECTION = init_detections("generic_detection")
+class ArgumentParser(ArgumentParser):
+    def error(self, message):
+        self.exit(2, 'Error: %s\n' % (message))
+
+def parse_args():
+    ARGV = None
+    ARG_PARSER = ArgumentParser(description="Qeeqbox/social-analyzer - API and Web App for analyzing & finding a person's profile across 300+ social media websites (Detections are updated regularly)", usage=SUPPRESS)
+    ARG_PARSER._action_groups.pop()
+    ARG_PARSER_REQUIRED = ARG_PARSER.add_argument_group("Required Arguments")
+    ARG_PARSER_REQUIRED.add_argument("--cli", action="store_true", help="Turn this CLI on", required=True)
+    ARG_PARSER_REQUIRED.add_argument("--username", help="E.g. johndoe, john_doe or johndoe9999", metavar="", required=True)
+    ARG_PARSER_OPTIONAL = ARG_PARSER.add_argument_group("Optional Arguments")
+    ARG_PARSER_OPTIONAL.add_argument("--websites", help="Website or websites separated by space E.g. youtube, tiktok or tumblr", metavar="", default="all")
+    ARG_PARSER_OPTIONAL.add_argument("--mode", help="Analysis mode E.g.fast -> FindUserProfilesFast, slow -> FindUserProfilesSlow or special -> FindUserProfilesSpecial", metavar="", default="fast")
+    ARG_PARSER_OPTIONAL.add_argument("--output", help="Show the output in the following format: json -> json output for integration or pretty -> prettify the output", metavar="", default="pretty")
+    ARG_PARSER_OPTIONAL.add_argument("--options", help="Show the following when a profile is found: link, rate, title or text", metavar="", default="")
+    ARG_PARSER_OPTIONAL.add_argument("--method", help="find -> show detected profiles, get -> show all profiles regardless detected or not, both -> combine find & get", metavar="", default="all")
+    ARG_PARSER_OPTIONAL.add_argument("--filter", help="filter detected profiles by good, maybe or bad, you can do combine them with comma (good,bad) or use all", metavar="", default="good")
+    ARG_PARSER_OPTIONAL.add_argument("--profiles", help="filter profiles by detected, unknown or failed, you can do combine them with comma (detected,failed) or use all", metavar="", default="detected")
+    ARG_PARSER_OPTIONAL.add_argument("--extract", action="store_true", help="Extract profiles, urls & patterns if possible", required=False)
+    ARG_PARSER_OPTIONAL.add_argument("--metadata", action="store_true", help="Extract metadata if possible (pypi QeeqBox OSINT)", required=False)
+    ARG_PARSER_OPTIONAL.add_argument("--trim", action="store_true", help="Trim long strings", required=False)
+    ARG_PARSER_LIST = ARG_PARSER.add_argument_group("Listing websites & detections")
+    ARG_PARSER_LIST.add_argument("--list", action="store_true", help="List all available websites")
+    ARGV = ARG_PARSER.parse_args()
+    return ARGV
+
+
+def init_logic():
+    global LANGUAGES_JSON
+    global SITES_DUMMY
+    global WEBSITES_ENTRIES
+    global SHARED_DETECTIONS
+    global GENERIC_DETECTION
+    print("[init] Detections are updated very often, make sure to get the most up-to-date ones")
+    makedirs(path.join(path.dirname(__file__), "data"), exist_ok=True)
+    LANGUAGES_JSON = load_file("languages.json", LANGUAGES_PATH, "https://raw.githubusercontent.com/qeeqbox/social-analyzer/main/data/languages.json")
+    SITES_DUMMY = load_file("sites.json", SITES_PATH, "https://raw.githubusercontent.com/qeeqbox/social-analyzer/main/data/sites.json")
+    WEBSITES_ENTRIES = init_detections("websites_entries")
+    SHARED_DETECTIONS = init_detections("shared_detections")
+    GENERIC_DETECTION = init_detections("generic_detection")
+
+def main_logic(ARGV=None):
+    global LANGUAGES_JSON
+    global SITES_DUMMY
+    if ARGV is None:
+        ARGV = parse_args()
+    if ARGV is not None:
+        if ARGV.cli:
+            init_logic()
+            if LANGUAGES_JSON is not None and SITES_DUMMY is not None:
+                if ARGV.list:
+                    setup_logger(argv=ARGV)
+                    list_all_websites()
+                elif ARGV.mode == "fast":
+                    if ARGV.username != "" and ARGV.websites != "":
+                        check_user_cli(ARGV)
 
-ARG_PARSER = ArgumentParser(description="Qeeqbox/social-analyzer - API and Web App for analyzing & finding a person's profile across 300+ social media websites (Detections are updated regularly)", usage=msg())
-ARG_PARSER._action_groups.pop()
-ARG_PARSER_REQUIRED = ARG_PARSER.add_argument_group("Required Arguments")
-ARG_PARSER_REQUIRED.add_argument("--cli", action="store_true", help="Turn this CLI on", required=True)
-ARG_PARSER_REQUIRED.add_argument("--username", help="E.g. johndoe, john_doe or johndoe9999", metavar="", required=True)
-ARG_PARSER_REQUIRED.add_argument("--websites", help="Website or websites separated by space E.g. youtube, tiktok or tumblr", metavar="", required=True)
-ARG_PARSER_REQUIRED.add_argument("--mode", help="Analysis mode E.g.fast -> FindUserProfilesFast, slow -> FindUserProfilesSlow or special -> FindUserProfilesSpecial", metavar="", required=True)
-ARG_PARSER_OPTIONAL = ARG_PARSER.add_argument_group("Optional Arguments")
-ARG_PARSER_OPTIONAL.add_argument("--output", help="Show the output in the following format: json -> json output for integration or pretty -> prettify the output", metavar="", default="")
-ARG_PARSER_OPTIONAL.add_argument("--options", help="Show the following when a profile is found: link, rate, title or text", metavar="", default="")
-ARG_PARSER_REQUIRED.add_argument("--method", help="find -> show detected profiles, get -> show all profiles regardless detected or not, both -> combine find & get", metavar="", default="all")
-ARG_PARSER_REQUIRED.add_argument("--filter", help="filter detected profiles by good, maybe or bad, you can do combine them with comma (good,bad) or use all", metavar="", default="all")
-ARG_PARSER_REQUIRED.add_argument("--profiles", help="filter profiles by detected, unknown or failed, you can do combine them with comma (detected,failed) or use all", metavar="", default="all")
-ARG_PARSER_REQUIRED.add_argument("--extract", action="store_true", help="Extract profiles, urls & patterns if possible", required=False)
-ARG_PARSER_REQUIRED.add_argument("--metadata", action="store_true", help="Extract metadata if possible (pypi QeeqBox OSINT)", required=False)
-ARG_PARSER_REQUIRED.add_argument("--trim", action="store_true", help="Trim long strings", required=False)
-ARG_PARSER_LIST = ARG_PARSER.add_argument_group("Listing websites & detections")
-ARG_PARSER_LIST.add_argument("--list", action="store_true", help="List all available websites")
-ARGV = ARG_PARSER.parse_args()
 
 if __name__ == "__main__":
-    if ARGV.output != "json":
-        print("[!] Detections are updated very often, make sure to get the most up-to-date ones")
-    if ARGV.cli:
-        if ARGV.list:
-            setup_logger()
-            list_all_websites()
-        elif ARGV.mode == "fast":
-            if ARGV.username != "" and ARGV.websites != "":
-                check_user_cli(ARGV)
+    main_logic()
